
% blocks.pro
% From "Logic, Programming and Prolog" by Ulf Nilsson and Jan Maluszynski

/* Consider a table with three distinct positions. On the table are  a number of blocks which may be stacked on top of each other. The aim is to move the blocks from a given start-state to a goal—state. Only blocks which are free (that is, with no other block on top of them) can be moved.
*/

% ?- path(X).

path(X) :-
   path( state(on(c,on(b,on(a,t))), t, t),    % c b a in pile 1
         state(t,t,on(c,on(a,on(b,t)))),      % c a b in pile 3
         [ state(on(c,on(b,on(a,t))),t,t)],
         X, 1, 8).    % added depth limit

/*

Shortest answer:

[ m(c,1,2),
  m(b,1,3),
  m(a,1,3),
  m(c,2,3)
]

*/

move( state(on(X,R),Y,Z),
      state(R,on(X,Y),Z), m(X,1,2)).    % move from first pile
move( state(on(X,R),Y,Z), 
      state(R,Y,on(X,Z)), m(X,1,3)). 

move( state(X,on(Y,R),Z), 
      state(on(Y,X),R,Z), m(Y,2,1)).   % move from second pile
move( state(X,on(Y, R),Z), 
      state(X,R,on(Y,Z)), m(Y,2,3)).

move( state(X,Y,on(Z,R)), 
      state(on(Z,X),Y,R), m(Z,3,1)).   % move from third pile
move( state(X,Y,on(Z,R)), 
      state(X,on(Z,Y),R), m(Z,3,2)).


path(To, To, Visited, [], _, MaxD).
path(State, To, Visited, [Mv|Path], D, MaxD) :-
  le(D, MaxD),
  nunify(State, To),
  move(State, NewState, Mv),
  not(member(NewState, Visited)),
  is(D1, add(D,1)),
  path(NewState, To, [NewState|Visited], Path, D1, MaxD).


not(X) :- call(X), !, fail.
not(X).

member(X, [X|_]).
member(X, [_|Rest]) :-
  member(X, Rest).
