
% colorMap.pro
% From "The Art of Prolog", Sterling & Shapiro
% Programs 14.4 and 14.5: Map coloring

% ?- testColor(test).
% ?- testColor(westEurope).


/* Test rig */

testColor(Name) :-
  map(Name,Map),
  colors(Name,Colors),
  colorMap(Map,Colors),
  printList(Map).

map(test,[region(a,A,[B,C,D]),     region(b,B,[A,C,E]), 
          region(c,C,[A,B,D,E,F]), region(d,D,[A,C,F]),
          region(e,E,[B,C,F]),     region(f,F,[C,D,E])]).
map(westEurope,
  [  region(portugal,P,[E]),  region(spain,E,[F,P]),
     region(france,F,[E,I,S,B,WG,L]),  region(belgium,B,[F,H,L,WG]),
     region(holland,H,[B,WG]), region(westGermany,WG,[F,A,S,H,B,L]),
     region(luxembourg,L,[F,B,WG]), region(italy,I,[F,A,S]),
     region(switzerland,S,[F,I,A,WG]), region(austria,A,[I,S,WG])]).


colors(X,[red,yellow,blue,white]).

printList([]) :- writeln("").
printList([X|Xs]) :- writeln(X), printList(Xs).



/*
colorMap(Map,Colors) :-
    Map is colored with Colors, so that no two neighbors have the same
    color. The map is represented as an adjacency-list of regions
    region(Name,Color,Neighbors), where Name is the name of the
    region, Color is its color, and Neighbors are the colors of the 
    neighbors. 
*/
colorMap([Region|Regions],Colors) :-
  colorRegion(Region,Colors),
  colorMap(Regions,Colors).
colorMap([],Colors).


/*
   colorRegion(Region,Colors) :-
    Region and its neighbors are colored using Colors so that the
    region's color is different from the color of any of its neighbors.
*/
colorRegion(region(Name,Color,Neighbors),Colors) :-
  select(Color,Colors,Colors1),
  members(Neighbors,Colors1).


select(X,[X|Xs],Xs).
select(X,[Y|Ys],[Y|Zs]) :- select(X,Ys,Zs).


members([X|Xs],Ys) :- member(X,Ys), members(Xs,Ys).
members([],Ys).

member(X, [X|_]).
member(X, [Y|Rest]) :- member(X, Rest).


