
% crypto.pro
% Bratko, Fig 7.2
% Solving crypotarithmetic puzzles

/*  some puzzles

?- crypto([D,O,N,A,L,D],[G,E,R,A,L,D],[R,O,B,E,R,T]).

The solution is unique
{D=5, O=2, N=6, A=4, L=8, G=1, E=9, R=7, B=3, T=0}

526485      DONALD
197485      GERALD
-—--—-+      —--—- +
723970      ROBERT



?- crypto( [0,S,E,N,D], [0,M,O,R,E], [M,O,N,E,Y]), neq(M,0).


The solution is unique if M=/=0, that is 
{S=9, E=5, N=6, D=7, M=1, O=0, R=8, Y=2}

 9567     SEND
 1085     MORE
 ———- +   -——— +
10652    MONEY


*/


crypto(N1, N2, N) :-     % Numbers represented as lists of digits
  sums(N1, N2, N,
        0, 0,             % Carries from right and to left both 0
       [0,1,2,3,4,5,6,7,8,9], _). % All digits available


sums([], [], [], 0, 0, Digits, Digits).
sums([D1|N1], [D2|N2], [D|N], C1, C, Digs1, Digs) :-
  sums( N1, N2, N, C1, C2, Digs1, Digs2),
  digitSum( D1, D2, C2, D, C, Digs2, Digs). 


digitSum( D1, D2, C1, D, C, Digs1, Digs) :-
  select( D1, Digs1, Digs2),   % Select an available digit for D1
  select( D2, Digs2, Digs3),   % Select an available digit for D2
  select( D, Digs3, Digs),     % Select an available digit for D
  is(S, add(D1, add(D2,C1))),
  is(D, mod(S,10)),
  is(C, div(S,10)).


select( A, L, L) :-
  nonvar(A), !.      % A already instantiated
select( A, [A|L], L).
select( A, [B|L], [B|L1] ) :-
  select( A, L, L1).