
% flights.pro
% Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

% A Flight Route Planner
% from Bratko, section 4.4

% ?- flight(london, ljubljana, Day, _, _, _).
% ?- route( ljubljana, edinburgh, th, R).



flight(Placel, Place2, Day, FNum, DeptTime, ArrTime) :-
  timetable(Placel, Place2, Flightlist),
  member( trip(DeptTime, ArrTime, FNum, Daylist), Flightlist),
  flyday(Day, Daylist).

member(X, [X|L]).
member(X, [Y|L]) :-
member(X, L).

flyday(Day, Daylist) :-
  member(Day, Daylist).
flyday(Day, alldays) :-
  member(Day, [mo,tu,we,th,fr,sa,su]).

route(P1, P2, Day, [fly(P1, P2, FNum, DeptTime)]) :- 
  % Direct flight
  flight(P1, P2, Day, FNum, DeptTime, _).
route(P1, P2, Day, [fly(P1, P3, FNum1, Dep1) | Route]) :- 
  % Indirect connection
  route(P3, P2, Day, Route),
  flight(P1, P3, Day, FNum1, Dep1, Arr1),
  deptime(Route, Dep2),
  transfer(Arr1, Dep2).

deptime([fly(Pl, P2, FNum, Dep)|_], Dep).

transfer(t(Hours1,Mins1), t(Hours2,Mins2)) :-
  is(Diff, add(mult(60,sub(Hours2,Hours1)), sub(Mins2,Mins1))),
  ge(Diff, 40).
  % (60*(Hours2 - Hoursl) + (Mins2 - Minsl)) >= 40.


% A Flight Database

timetable(edinburgh, london,
  [ trip(t(9,40), t(10,50), ba4733, alldays),
    trip(t(13,40), t(14,50), ba4773, alldays),
    trip(t(19,40), t(20,50), ba4833, [mo,tu,we,th,fr,su]) ]).

timetable(london, edinburgh,
  [ trip(t(9,40), t(10,50), ba4732, alldays),
    trip(t(11,40), t(12,50), ba4752, alldays),
    trip(t(18,40), t(19,50), ba4822, [mo,tu,we,th,fr]) ]).
timetable(london, ljubljana,
  [ trip(t(13,20), t(16,20), yu201, [fr]),
    trip(t(13,20), t(16,20), yu213, [su]) ]).
timetable(london, zurich,
  [ trip(t(9,10), t(11,45), ba614, alldays),
    trip(t(14,45), t(17,20), sr805, alldays) ]).
timetable(london, milan,
  [ trip(t(8,30), t(11,20), ba510, alldays),
    trip(t(11,00), t(13,50), az459, alldays) ]).

timetable(ljubljana, zurich,
  [ trip(t(11,30), t(12,40), yu322, [tu,th]) ]).
timetable(ljubljana, london,
  [ trip(t(11,10), t(12,20), yu200, [fr]),
    trip(t(11,25), t(12,20), yu212, [su]) ]).

timetable(milan, london,
  [ trip(t(9,10), t(10,00), az458, alldays),
    trip(t(12,20), t(13,10), baSll, alldays) ]).
timetable(milan, zurich,
  [ trip(t(9,25), t(10,15), sr621, alldays),
    trip(t(12,45), t(13,35), sr623, alldays) ]).

timetable(zurich, ljubljana,
  [ trip(t(13,30), t(14,40), yu323, [tu,th]) ]).
timetable(zurich, london,
  [ trip(t(9,00), t(9,40), ba613, [mo,tu,we,th,fr,sa]),
    trip(t(16,10), t(16,55), sr806, [mo,tu,we,th,fr,su]) ]).
timetable(zurich, milan,
  [ trip(t(7,55), t(8,45), sr620, alldays) ]).

