
% jugs.pro
% From "Logic, Programming and Prolog" by Ulf Nilsson and Jan Maluszynski

/* Two water jugs are given, a 4-gallon and a 3-gallon jug. Neither of them has any type of marking on it. There is an infinite supply of water (a tap?) nearby. How can you get exactly 2 gallons of water into the 4-gallon jug? Initially both jugs are empty.

 * empty the 4-gallon jug if it is not already empty;
 * empty the 3-gallon jug if it is not already empty;
 
 * fill up the 4-gallon jug if it is not already full;
 * fill up the 3-gallon jug if it is not already full;
  
 * if there is enough water in the 3—gallon jug, use it to fill up the 4—gallon jug until it is full;
 * if there is enough water in the 4-gallon jug, use it to fill up the 3—gallon jug until it is full;
 
 * if there is room in the 4-gallon jug, pour all water from the 3-gallon jug into it;
 * if there is room in the 3-gallon jug, pour all water from the 4-gallon jug into it.
*/

% ?- path(X).

action(jugs(X,Y), jugs(0,Y)) :- 
  gt(X,0).
action(jugs(X,Y), jugs(X,0)) :- 
  gt(Y,0).
action(jugs(X,Y), jugs(4,Y)) :- 
  lt(X,4).
action(jugs(X,Y), jugs(X,3)) :- 
  lt(Y,3).
action(jugs(X,Y), jugs(4,Z)) :- 
  lt(X,4), is(Z,sub(Y, sub(4,X))), ge(Z,0).
action(jugs(X,Y), jugs(Z,3)) :- 
  lt(Y,3), is(Z,sub(X, sub(3,Y))), ge(Z,0).
action(jugs(X,Y), jugs(Z,0)) :- 
  gt(Y,0), is(Z,add(X,Y)), le(Z,4).
action(jugs(X,Y), jugs(2,Z)) :- 
  gt(X,0), is(Z,add(X,Y)), le(Z,3).
        

path(X) :-
  path(jugs(0,0), [jugs(0,0)], X).

path(jugs(2,X), Visited, Visited).
path(State, Visited, Path) :-
  action(State, NewState),
  not(member(NewState, Visited)),
  path(NewState, [NewState|Visited], Path).


not(X) :- call(X), !, fail.
not(X).

member(X, [X|_]).
member(X, [_|Rest]) :-
  member(X, Rest).
