
% math.pro
% "Programming in Prolog", Clocksin & Mellish


% ?- findPrimes(100, Ps).
% ?- gcd(27,15,D).
% ?- lcm(8,9,M).
% ?- ack(1,2,A).
% ?- ack(3,1,A).
% ?- ack(4,3,A).
% ?- fib(7,F).
% ?- fiba(7,F).



findPrimes(N, Ps) :-
  genNums(2, 100, Nums),
  primes(Nums, [], Ps).


genNums(Max, Max, []).
genNums(N, Max, [N|Ls]) :-
  lt(N, Max),
  is(N1, add(N,1)),
  genNums(N1, Max, Ls).
  

primes([], _, []).
primes([H|T], P, [H|Ps]) :-
  legal(H, P),
  !,
  primes(T, [H|P],Ps).
primes([H|T], P, Ps) :- 
  primes(T, P, Ps).


/* X is legal in L if X is not divided by any member of L */
legal(X, []).
legal(X, [H|_]) :-
  is(0, mod(X, H)),
  !,
  fail.
legal(X, [_|L]) :- 
  legal(X, L).


% ------------------------

gcd(I, 0, I) :- !.
gcd(I, J, K) :- 
  is(R, mod(I, J)), 
  gcd(J, R, K).

lcm(I, J, K) :- 
  gcd(I, J, R), 
  is(K, div(mult(I,J),R)).


/*
ack(X,Y,A) :-
	A is the value of Ackermann's function for X and Y.
e.g. ack(1,2,A), A == 4
     ack(3,1,A), A == 13
     ack(4,3,A), A == ??
*/

ack(0, N, N1) :- 
  !, is(N1,add(N,1)).

ack(M1, 0, Val) :- 
  !, is(M, sub(M1,1)),
  ack(M, 1, Val).

ack(M1, N1, Val) :-
  !, is(N, sub(N1,1)),
	ack(M1, N, Val1), 
  is(M, sub(M1,1)),
  ack(M, Val1, Val).


% --------------------------------


fib(0,0).
fib(1,1).
fib(N,F) :-
  gt(N, 1),
  is(N1, sub(N,1)), fib(N1,F1), 
  is(N2, sub(N,2)), fib(N2,F2),
  is(F, add(F1,F2)).


fiba(N, F) :-
  fibacc(N, 2, 1, 0, F).

fibacc(N, N, F1, F2, F):- 
  is(F, add(F1,F2)).
fibacc(N, Count, F1, F2, F):- 
  lt(Count, N), 
  is(Count1, add(Count, 1)), 
  is(F1New, add(F1,F2)), 
  fibacc(N, Count1, F1New, F1, F).

