
% mc.pro
% Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

% the missionaries and cannibals problem
% http://www.cse.unsw.edu.au/~billw/cs9414/notes/mandc/mandc.html
% https://en.wikipedia.org/wiki/Missionaries_and_cannibals_problem

%  ?- mc(state(3,3,left), [state(3,3,left)], Path), printList(Path).


% mc(CurrState, Visited, Path)

mc(state(0,0,right), _, []).
mc(CurrState, Visited, [Move|Moves]) :-
  newState(CurrState, NextState),
  nonMember(NextState, Visited),
  makeMove(CurrState, NextState, Move),
  mc(NextState, [NextState|Visited], Moves).


% makeMove(State1, State2, Move) builds the 
% move(-,-,-) that gets you
% from State1 to State2.

makeMove(state(M1,C1,left), state(M2,C2,right), 
      move(M,C,right)) :-
  is(M, sub(M1, M2)),   % M is M1 - M2,
  is(C, sub(C1, C2)).   % C is C1 - C2.

makeMove(state(M1,C1,right), state(M2,C2,left), 
      move(M, C, left)) :-
  is(M, sub(M1, M2)),   % M is M1 - M2,
  is(C, sub(C1, C2)).   % C is C1 - C2.


% carry(X,Y) is true if the boat can carry 
% X missionaries and Y cannibals.

carry(2, 0).
carry(1, 0).
carry(1, 1).
carry(0, 1).
carry(0, 2).


% legal(X, Y) is true if it is safe for the 
% missionaries to have X
% missionaries and Y cannibals together on a 
% river bank (left or right).

legal(X, X).
legal(3, X).
legal(0, X).


% newState(State1, State2) is true if it is 
% possible to get from State1 to State2.

newState(state(M1,C1,left), state(M2,C2,right)) :-
  carry(M, C),
  le(M, M1),    % M <= M1,
  le(C, C1),    % C <= C1,
  is(M2, sub(M1, M)),   % M2 is M1 - M,
  is(C2, sub(C1, C)),   % C2 is C1 - C,
  legal(M2, C2).

newState(state(M1,C1,right), state(M2,C2,left)) :-
  carry(M, C),
  is(M2, add(M1, M)),   % M2 is M1 + M,
  is(C2, add(C1, C)),   % C2 is C1 + C,
  le(M2, 3),   % M2 <= 3,
  le(C2, 3),   % C2 <= 3,
  legal(M2, C2).


nonMember(_,[]).
nonMember(X,[Y|T]):-
   nunify(X, Y),  %  X\=Y,
   nonMember(X,T).


printList([]) :- writeln('').
printList([P|Rest]) :-
  writeln(P),
  printList(Rest).



