
% nqueens.pro
% Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

% The N queens problem
% From "The Art of Prolog", Sterling & Shapiro, 
% Section 14.1 (v)

% Nicely explained at https:%stackoverflow.com/questions/56221482/hints-to-understand-splendid-program-to-solve-queens

% ?- queens(4, Qs).
% ?- queens(8, Qs).


/* -------- Meaning of Variables ------
  N, M  ... Size of the board
  I, J  ... Number of the row current queen is on
  Qs, L ... List of length N used to represent the solution
  Cs ... Column as a list of fields of length N
  Us ... Up-Diagonal as an open list of fields
  Ds ... Down-Diagonal as an open list of fields
*/

queens(N,Qs):- 
  genList(N,Qs), 
  placeQueens(N,Qs,_,_).

genList(0,[]).
genList(N,[_|L]):-
  gt(N,0), 
  is(N1, sub(N, 1)),
  genList(N1,L).

placeQueens(0,_,_,_).
placeQueens(I,Cs,Us,[_|Ds]):-
  gt(I,0),
  is(I1, sub(I, 1)),
  placeQueens(I1,Cs,[_|Us],Ds),
  placeQueen(I,Cs,Us,Ds).

% placeQueen(Queen,Column,Up-diagonal,Down-diagonal) places a single queen
placeQueen(I,[I|_],[I|_],[I|_]).
placeQueen(I,[_|Cs],[_|Us],[_|Ds]):-
  placeQueen(I,Cs,Us,Ds).


