
% parents.pro
% Andrew Davison, June 2020, ad@fivedots.coe.psu.ac.th


% parentOf/2 predicate (made of 6 facts)
parentOf(kim,holly).     % kim is the parent of holly
parentOf(margaret,kim).
parentOf(margaret,kent).
parentOf(esther,margaret).
parentOf(herbert,margaret).
parentOf(herbert,jean).


% livesAt/3 predicate (made of 8 facts)
livesAt(margaret, 9, 'Bar Lane').
livesAt(kim, 37, 'Foo Street').
livesAt(holly, 37, 'Foo Street').
livesAt(esther, 9, 'Bar Lane').
livesAt(herbert, 23, 'PSU Village').
livesAt(kent, 9, 'Bar Lane').
livesAt(bill, 23, 'PSU Village').
livesAt(john, 9, 'Bar Lane').


% livesWith/2 predicate (made of 1 rule)
livesWith(X, Y) :-
  livesAt(X, No, Addr), livesAt(Y, No, Addr).


% livesWithParent/2
livesWithParent(X, Y) :-
  livesWith(X, Y), parentOf(Y, X).


% staff/3 predicate (made of 3 facts)
staff( name(mickey, mouse),  address(123, 'Fantasy Way'), 73).
staff( name(bat, man),  address(321, 'Cavern Ave'), 54).
staff( name(wonder, woman),  address(987, 'Truth Way'), 39).


append([],L,L).
append([H|T],L2,[H|L3]) :- append(T,L2,L3). 

member(X, [X|_]).
member(X, [Y|Rest]) :- member(X, Rest).

length([], 0).
length([X|Rest], Len) :-
  length(Rest, LenRest),
  is(Len, add(LenRest,1)).

nonMember(_,[]).
nonMember(X,[Y|T]):-
   nunify(X, Y),  %  X\=Y,
   nonMember(X,T).


not(X) :- call(X), !, fail.
not(X).



% ======= queries ======


% ?- parentOf(margaret, kent).
% ?- parentOf(fred, pebbles).
% ?- parentOf(P, jean).
% ?- parentOf(P, esther).
% ?- parentOf(margaret,X), parentOf(X,holly).
% ?- parentOf(margaret,Child).
% ?- parentOf(X, Y).


% ?- livesAt(margaret, No, Addr).
% ?- livesAt(X, 9, 'Bar Lane').

% ?- livesWith(bill, herbert).
% ?- livesWith(kim, X).


% ?- livesWithParent(holly, kim).
% ?- livesWithParent(kim, holly).


% ?- member(1, [2, 1,3]).
% ?- member(j, [a,n,d,y]).
% ?- member(X,[j,i,m]).


% ?- length([a,b,c,d], L).
% ?- length([1,2,3], 4).
% ?- length([a,b,c,d], L).


% ?- is(X, add(1, mult(2,3))).


% ?- append([1,2],[3,4],Z).
% ?- append(X,[3,4],[1,2,3,4]).
% ?- append(X,Y,[1,2,3]).


% ?- nonMember(4,[1,2,3]).
% ?- nonMember(1,[2,1,3]).
% ?- not( member(4,[1,2,3])).
% ?- not( member(1,[2,1,3])).


% ?- staff(Name, address(123, 'Fantasy Way'), 73).
% ?- staff(Name, address(_, 'Fantasy Way'), _).
% ?- staff( name(First, Last), address(_, 'Fantasy Way'), _).

