
// roman.pro

/*
  Convert years into roman numerals.

  https://www.rosettacode.org/wiki/Roman_numerals/Encode#Prolog
  https://cs.stackexchange.com/questions/7777/is-the-language-of-roman-numerals-ambiguous

  http://www.onlineconversion.com/roman_numerals_advanced.htm


  Only between 1 and 4999

  1789 == MDCCLXXXIX
  2020 == MMXX
  110 == CX
  1666 == MDCLXVI
*/


% ?- roman(1789).
% ?- roman(2020).
% ?- roman(1666).




roman(Num) :-
  ge(Num, 1), lt(Num, 5000),
  roman(Num, Rs, [ r(1000,'M'), r(900,'CM'), r(500,'D'), r(400,'CD'),
                    r(100,'C'), r(90,'XC'), r(50,'L'), r(40,'XL'),  
                    r(10,'X'), r(9,'IX'), r(5,'V'), r(4,'IV'), r(1,'I') ]),
  writeList(Rs).


roman(0, [], _).
roman(Num, [R|Rest], Convs) :-
  reduce(Num, Num1, R, Convs),
  roman(Num1, Rest, Convs).
        

reduce(Num, Num1, R, [r(N,R)|_]) :-
  ge(Num,N), 
  is(Num1, sub(Num,N)).
reduce(Num, Num1, R, [_|Rest]) :- 
  reduce(Num, Num1, R, Rest).


writeList([]) :- 
  writeln('').
writeList([Ch|Chars]) :-
  write(Ch),
  writeList(Chars).