
% solve.pro
% Andrew Davison, ad@fivedots.coe.psu.ac.th, July 2020

% A simple Prolog meta-interpreter

% ?- sol1( fact(2)).
% ?- sol1( fact(X)).

% ?- sol2( length([1,2,3], X)).


% ?- sol3( app(X,Y,[1,2,3])).
% ?- sol3( app([],[1,2,3],L)).


% ---------------- program setups ----------------


sol1(Goal) :-
  unify(Program,
     [ fact(1),    
       fact(2)
     ]),
  solve(Program, [ Goal ]).



sol2(Goal) :-
  unify(Program,
     [ length([], 0),
       rule( length( cons(v(1),v(2)), v(3)),
              [ length(v(2),v(4)),
                is(v(3), add(v(4), 1)) ] )
              % length([_|Rest], N) :-
              %   length(Rest, N1), is(N, add(N1, 1)).
     ]),
  solve(Program, [ Goal ]).



sol3(Goal) :-
  unify(Program,
     [ app([], v(1), v(1)), % append([],L,L)
       rule( app( cons(v(1),v(2)), v(3),
                     cons(v(1),v(4)) ),
               [ app(v(2),v(3),v(4)) ] )
             % append([H|T],L2,[H|L3]) :- append(T,L2,L3). 
     ]),
  solve(Program, [ Goal ]).



% ----------------- solve --------------------------

solve(Program, []).  
solve(Program, [Goal|Goals]) :- 
  solveGoal(Program, Goal, Bs),
  append(Bs, Goals, NGoals),
  solve(Program, NGoals).



solveGoal(Program, Goal, Bs) :-
  clause(Program, Goal, Bs).  % user-defined preds
solveGoal(Program, Goal, []) :-
  call(Goal).                 % built-ins (and preds defined here)



clause([rule(H,Bs)|Cs], Goal, NBs) :-     % select a rule
  copyTerm(rule(H,Bs), rule(Goal,NBs)).
clause([Fact|Cs], Goal, []) :-            % select a fact
  nunify(Fact, rule(_,_)),
  copyTerm(Fact, Goal).
clause([Cl|Cs], Goal, Bs) :-     % or continue looking
  clause(Cs, Goal, Bs).



% http://kti.ms.mff.cuni.cz/~bartak/prolog/data_struct.html
copyTerm(A, B):-
  cp(A,[], B, _).
   
cp([], Vars, [], Vars) :- !.
cp([X|Y], Vars, [NX|NY], Vars) :- 
  !, cp(X, Vars, NX, Vars1),
  cp(Y, Vars1, NY, NVars).
cp(v(N), Vars, NV, NVars) :-     % meta-level var --> Prolog var
   !, mapVar(N, Vars, NV, NVars).
cp(A, Vars, A, Vars) :-
   atomic(A), !.
cp(cons(X,Y), Vars, [NX|NY], NVars) :-
  !, cp(X, Vars, NX, Vars1),
  cp(Y, Vars1, NY, NVars).
cp(Term, Vars, NTerm, NVars) :-
  !, univ(Term, F, Args),    % decompose term
  cpArgs(Args, Vars, NArgs, NVars),
  univ(NTerm, F, NArgs).  % construct copy
   


cpArgs([H|T], Vars, [NH|NT], NVars) :-
   cp(H, Vars, NH, Vars1),
   cpArgs(T, Vars1, NT, NVars).
cpArgs([], Vars, [], Vars).



mapVar(Num, [map(N,V)|T], NV, [map(N,V)|NT]) :-
   neq(Num, N),  !,        % different vars
   mapVar(Num, T, NV, NT).
mapVar(Num, [map(N,V)|T], V, [map(N,V)|T]) :-
   eq(Num, N), !.        % same vars
mapVar(Num, [], NV, [map(Num,NV)]).


append([],L,L).
append([H|T],L2,[H|L3]) :- append(T,L2,L3). 


atomic(X) :- atom(X).
atomic(X) :- integer(X).

