
// Builtin.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

/* builtin predicates.
*/

public enum Builtin
{
  WRITELN("writeln", -10),
      // every builtin has a name and a negative opcode
  WRITE("write", -11),
  ATOM("atom", -12),
  INTEGER("integer", -13),
  VAR("var", -14),
  NONVAR("nonvar", -15),
  FAIL("fail", -16),
                                                              
  IS("is", -20),
      // for ops understood by is, see BuWAM.java 
  READLN("readln", -21),
  READ("read", -22),

  EQ("eq", -30),    // ==
  LT("lt", -31),    // <
  LE("le", -32),    // <=
  GT("gt", -33),    // >
  GE("ge", -34),    // <=
  NEQ("neq", -35),  // !=retur

  CUT("cut", -40),  // !
  UNIFY("unify", -41),  // =
  NOT_UNIFY("nunify", -42),  // \=

  CALL("call", -50),
  UNIV("univ", -51);  // =..



  private String predName;
  private int predCode;   
    /* use a negative number, but not -1 which is used
       elsewhere to mean an error */


  Builtin(String predName, int predCode)
  {  this.predName = predName;
     this.predCode = predCode;
  }


  public String getName()
  {  return predName; }

  public int getCode()
  {  return predCode; }


  public static boolean contains(String nm) 
  // is nm the name of a builtin?
  {
    for (Builtin b : Builtin.values())
      if (b.predName.equalsIgnoreCase(nm))
        return true;
    return false;
  }  // end of contains()


  public static boolean contains(int code) 
  // is this code number for a builtin?
  {
    for (Builtin b : Builtin.values())
      if (b.predCode == code)
        return true;
    return false;
  }  // end of contains()


  public static Builtin fromName(String nm)
  // convert this name to a Builtin object
  {
    for (Builtin b : Builtin.values()) {
      if (b.predName.equalsIgnoreCase(nm))
        return b;
    }
    // System.out.println("No builtin for \"" + nm + "\"");
    return null;
  }  // end of fromName()



  public static Builtin fromCode(int code)
  // convert this code number to a Builtin object
  {
    for (Builtin b : Builtin.values()) {
      if (b.predCode == code)
        return b;
    }
    // System.out.println("No builtin with code \"" + code + "\"");
    return null;
  }  // end of fromCode()



}  // end of Builtin class
