
// ChoicePoint.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020


/* A choice point contains all the information necessary to 
   restore an earlier state in the event of backtracking. 

   One is created when calling a predicate if it
   has more than one clause which can potentially 
   match the call. See WAM.try_me_else() and WAM.backtrack()

   ChoicePoint objects are stored in a stack in the BuWAM class
*/


import java.io.*;
import java.util.*;



public class ChoicePoint
{
  // public so the data can be accessed without get() methods;
  // think of it as a record

  public ArrayList<Var> args;     // the ARG vars

  public int envIdx;              // index into environment list
                                  // for the 'previous' clause's PERM vars;
                                  // called CE in the original WAM

  public int cp;                  // continuation pointer
                                  // (i.e. the return address)

  public int trailIdx;            // current trail index for undoing
                                  // variable bindings;
                                  // called TR in the original WAM

  public int elseLabel;           // address of next clause to try
                                  // during backtracking;
                                  // called BP in the original WAM

  public ChoicePoint cutPoint;    // used to backtrack over a cut;
                                  // called B0 in the original WAM


  // diagnostics data
  public String label;            // label of this clause

  public String callNodeNm;       // name of callgraph node for this
                                  // clause
  public int tryCount;            // nodesCount of the current try



  public ChoicePoint(ArrayList<Var> vargs)
  {
    args = new ArrayList<>();
    for (int i = 0; i < vargs.size(); i++)
      args.add(new Var(vargs.get(i)));

  } // end of ChoicePoint()



} // end of class ChoicePoint

