
// OpWam.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

/* WAM operations

   For details on the WAM, see 
   "Warren's Abstract Machine: A Tutorial Reconstruction" 
   by Hassan Hvt-Kaci, available at 
   https://github.com/a-yiorgos/wambook
*/


public enum OpWam
{
  // PUT_VARIABLE("put_variable"),
  PUT_VALUE("put_value"),
  PUT_CONSTANT("put_constant"),
  CREATE_VARIABLE("create_variable"),  // NEW

  GET_VARIABLE("get_variable"),
  GET_VALUE("get_value"),
  GET_CONSTANT("get_constant"),

  UNIFY_VARIABLE("unify_variable"),    // two-arg version
  UNIFY_LIST("unify_list"),            // NEW
  UNIFY_STRUC("unify_struc"),          // NEW

  ALLOCATE("allocate"),
  DEALLOCATE("deallocate"),
  CALL("call"),
  PROCEED("proceed"),
  HALT("halt"),         // NEW

  // for the cut
  GET_LEVEL("get_level"),
  CUT("cut"),

  TRY_ME_ELSE("try_me_else"),
  RETRY_ME_ELSE("retry_me_else"),
  TRUST_ME("trust_me"),

  NO_OP("noop");


  private String opName;


  OpWam(String opName)
  {  this.opName = opName; }


  public String getName()
  {  return this.opName; }


  public static OpWam fromString(String opName)
  {
    for (OpWam b : OpWam.values()) {
      if (b.opName.equalsIgnoreCase(opName))
        return b;
    }
    System.out.println("No WAM op for \"" + opName + "\"; using NO_OP");
    return NO_OP;
  }  // end of fromString()


}  // end of OpWam class
