
// Trail.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

/* an undo-list for variable bindings
   that need to be undone during backtracking

*/

import java.io.*;
import java.util.*;



public class Trail
{
  private ArrayList<Var> contents;



  public Trail()
  {  contents = new ArrayList<>(); }


  public void reset()
  {  contents.clear();  }


  public int getLength()
  {  return contents.size(); }


  public void add(Var v)
  {  contents.add(v);  }



  public void rollbackTo(int pos)
  /* "undoing" means unbinding a variable by making it a REF
      again, and making its reference point back to itself
  */
  {
    for (int i = contents.size()-1; i >= pos; i--) {
      Var v = contents.get(i);
      if (v != null) {
        v.tag = Var.ValType.REF;
        v.ref = v;    // an unbound var points to itself
        v.refersToArg = -1;
      }
    }
  }  // end of rollbackTo()


} // end of class Trail

