
// VarType.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

/* 
  PERM variables (Yxx) are those used in a clause, and stored in an Env
  in BuWAM.java;

  ARG are the argument variables (Axx), stored in args in BuWAM.java;

  QUERY are the query variables (Qxx) stored in queryVars in BuWAM.java

*/


public enum VarType
{

  PERM("Y"),
  ARG("A"),
  QUERY("Q"),
  NONE("");    // error case; should not occur


  private String id;


  VarType(String id)
  {  this.id = id; }


  public String getId()
  {  return this.id; }


  public static VarType fromId(String id)
  {
    for (VarType b : VarType.values()) {
      if (b.id.equalsIgnoreCase(id))
        return b;
    }
    return NONE;
  }  // end of fromId()



  public static VarType getType(String label)
  // e.g. from V23 return PERM
  {
    if ((label == null) || label.equals(""))
      return NONE;
    char ch = label.toUpperCase().charAt(0);
    switch (ch) {
      case 'Y':  return PERM;
      case 'A':  return ARG;
      case 'Q':  return QUERY;
      default:   return NONE;
    }
  } // end of getType()


  public static int getIndex(String label)
  // e.g. from V23 return 23
  {
    if ((label == null) || label.equals(""))
      return -1;

    String numStr = label.replaceAll("[^0-9]", "");
    if (numStr.equals(""))
      return -1;

    int idx;
    try {
      idx = Integer.parseInt(numStr);
    }
    catch (NumberFormatException e) 
    { idx = -1; }
    return idx;
  }  // end of getIndex()
    

}  // end of VarType class
