
// WAMProgram.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, June 2020

/* Used to convert a ParseNode tree into the code for
   a WAM program

   The code is stored as a list of WAM clauses 
   (see WAMClause.java). Each clause has a label which
   is also maintained in a separate list.

   The code is saved to a file by calling toString in main()
   of BuProlog.java

   WAMProgram is called by BuProlog.java while WAMCode is called 
   by BuWAM.java. Both of them use WAMStatement.
*/

import java.util.*;


public class WAMProgram
{

  private ArrayList<WAMClause> clauses = new ArrayList<>();
  private ArrayList<String> labels = new ArrayList<>();

  private boolean debugOn;


  public WAMProgram(ParseNode prog, boolean debugOn)
  {
    this.debugOn = debugOn;
    ParseNode t = prog;
    while (t != null) {   // a series of clauses
      if (t.type == ParseType.CLAUSES) {
        WAMClause cl = new WAMClause(t.head, debugOn);
        String label = cl.getLabel();
        if (labels.contains(label))
          System.out.println("Error: Multiple occurrence of label \"" + 
                               label + "\".");
        else {
          labels.add(label);
          clauses.add(cl);
        }
      }
      else 
        System.out.println("Error: expected a CLAUSES node");
      t = t.tail;  // more clauses
    } 
  }


  public String toString()
  // return the WAM code as a string
  {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < clauses.size(); i++)
      sb.append( clauses.get(i).toString() );
    return sb.toString();
  } // end of toString()




} // end of class WAMProgram
